#include "pwm_ctrl.h"
#include "led_ctrl.h"
#ifdef LEVEL
#include "portg_ctrl.h"
#endif
#ifdef VOLUME
#include "adc_ctrl.h"
#endif
#include "audio.h"
#include <stdlib.h>

typedef struct stereo {
	short l;
	short r;
} t_stereo;

static int decode_get(t_stereo *decode_data);
#ifdef LEVEL
static unsigned char level2bar(int data);
#endif /* LEVEL */

static t_stereo ringbuf[1152 + 1];
static unsigned int decode_putp = 0, decode_getp = 0;

#define RINGBUF_SIZE_MAX (sizeof(ringbuf)/sizeof(t_stereo))
#define p_inc(p, max) (((p + 1) >= max) ? 0 : (p + 1))

/* Oobt@ɃfR[hf[^ */
int decode_put(short left, short right)
{
	unsigned int putp = p_inc(decode_putp, RINGBUF_SIZE_MAX);
	int r = PUT_NG;

	if(putp != decode_getp)
	{
		ringbuf[decode_putp].l = left;
		ringbuf[decode_putp].r = right;

		decode_putp = putp;
		r = PUT_OK;
	}
	return r;
}

/* Oobt@fR[hf[^ */
static int decode_get(t_stereo *decode_data)
{
	int r = GET_NG;

	if(decode_putp != decode_getp)
	{
		decode_data->l = ringbuf[decode_getp].l;
		decode_data->r = ringbuf[decode_getp].r;

		decode_getp = p_inc(decode_getp, RINGBUF_SIZE_MAX);
		r = GET_OK;
	}
	return r;
}

/* Oobt@ */
int decode_empty(void)
{
	int r = EMPTY_NG;

	if(decode_putp == decode_getp)
	{
		r = EMPTY_OK;
	}
	return r;
}

#ifdef LEVEL
static unsigned char level2bar(int data)
{
	unsigned char r;

	if(data > (0x8000/9 * 8))
	{
		r = 0xff;
	}
	else if(data > (0x8000/9 * 7))
	{
		r = 0x7f;
	}
	else if(data > (0x8000/9 * 6))
	{
		r = 0x3f;
	}
	else if(data > (0x8000/9 * 5))
	{
		r = 0x1f;
	}
	else if(data > (0x8000/9 * 4))
	{
		r = 0x0f;
	
	}else if(data > (0x8000/9 * 3))
	{
		r = 0x07;
	}
	else if(data > (0x8000/9 * 2))
	{
		r = 0x03;
	}
	else if(data > (0x8000/9 * 1))
	{
		r = 0x01;
	}
	else
	{
		r = 0;
	}
	return r;
}
#endif /* LEVEL */

void audio(void)
{
	t_stereo decode_data;

	#ifdef LEVEL
	static unsigned int x44 = 0, x10 = 0;
	static t_stereo level = {0, 0};
	#ifdef VOLUME
	static t_stereo volume = {0, 0};
	#endif /* VOLUME */
	#endif /* LEVEL */

	if(GET_OK == decode_get(&decode_data))
	{
		#ifdef VOLUME
		decode_data.l = (short)((decode_data.l * volume.l) / 256);
		decode_data.r = (short)((decode_data.r * volume.r) / 256);
		#endif /* VOLUME */

		pwm_set_1a((unsigned short)((decode_data.l / 256) + 128));
		pwm_set_1c((unsigned short)((decode_data.r / 256) + 128));

		#ifdef LEVEL
		if(++x44 >= 44)
		{
			x44 = 0;
			if(level.l < abs((int)decode_data.l))
			{
				level.l = (short)abs((int)decode_data.l);
			}
			if(level.r < abs((int)decode_data.r))
			{
				level.r = (short)abs((int)decode_data.r);
			}

			if(++x10 >= 10)
			{
				x10 = 0;

				portg_l((unsigned char)~level2bar((int)level.l));
				portg_h((unsigned char)~level2bar((int)level.r));

				level.l = 0;
				level.r = 0;

				#ifdef VOLUME
				if(ADC_OK == adc_complete())
				{
					volume.l = (short)(adc_get_ch0() / 256);
					volume.r = (short)(adc_get_ch1() / 256);
					adc_start();
				}
				#endif /* VOLUME */
			}
		}
		#endif /* LEVEL */

		led_off();
	}
	else
	{
		led_on();
	}
	return;
}
