#include <string.h>
#include <stdio.h>
#include "../fatfs/ff.h"
#include "play.h"
#ifdef USE_SRC
#include "src_ctrl.h"
#endif

extern int decode(unsigned char const *start, unsigned long length);

#define MP3_WORK_SIZE 0x3000	/* 1152Tv ~ 2byte ~ 2ch ~ 2t[ȏ */
static unsigned char mp3_work[MP3_WORK_SIZE];
static unsigned char *next_header;
FIL fobj;

static FRESULT get_header(void)
{
	FRESULT res;
	UINT size;

	res = f_read(&fobj, (void*)mp3_work, 10, &size);
	if(res != FR_OK) goto get_header_return;

	if(0 == strncmp((char*)mp3_work, "ID3", 3))
	{
		UINT id3_size =
			((*(mp3_work + 6) & 0x7f) << 21) +
			((*(mp3_work + 7) & 0x7f) << 14) +
			((*(mp3_work + 8) & 0x7f) << 7) +
			((*(mp3_work + 9) & 0x7f) << 0);

		res = f_read(&fobj, (void*)(mp3_work + 10), id3_size, &size);	/* ID3ver2^Oǂݔ΂ */
		if(res != FR_OK) goto get_header_return;

		res = get_header();	/* x */
		goto get_header_return;
	}

	if(0 == strncmp((char*)mp3_work, "RIFF", 4))
	{
		res = f_read(&fobj, (void*)(mp3_work + 10), 10, &size);	/* RIFF MP3wb_ǂݔ΂ */
		if(res != FR_OK) goto get_header_return;

		res = get_header();	/* x */
		goto get_header_return;
	}

get_header_return:
	return res;
}

int play(char* filename)
{
	FRESULT res;

	next_header = 0;

	res = f_open(&fobj, filename, FA_READ);
	if(res != FR_OK) goto play_return;

	res = get_header();
	if(res != FR_OK) goto play_return;

	if(mp3_work[0] != 0xff) goto play_return;
	if(mp3_work[1] & 0xf0 != 0xf0) goto play_return;
	next_header = mp3_work;

	#ifdef USE_SRC
	src_change_ifs((mp3_work[2] >> 2) & 0x03);	/* σTvO[gɂ͑Ήs */
	#endif

	decode((unsigned char const *)mp3_work, (unsigned long)MP3_WORK_SIZE);

	res = f_close(&fobj);
	if(res != FR_OK) goto play_return;

play_return:
	return (int)res;
}

void read_frame(unsigned int N)
{
	FRESULT res;
	UINT size;

	res = f_read(&fobj, (void*)(mp3_work + 10), N, &size);	/* 10oCgڈȍ~NoCg[Nɓǂݍ */
	if(res != FR_OK) goto read_frame_error;
	if(N != size) goto read_frame_error;

	next_header = mp3_work + N;
	return;

read_frame_error:
	*(int*)mp3_work = 0;
	next_header = mp3_work;
	return;
}

void copy_next_header(void)
{
	memcpy(mp3_work, next_header, 10);	/* ǂ݂ũt[̃wb_10oCgv擪ɃRs[ */
	return;
}

unsigned char* get_next_frame(void)
{
	return (unsigned char*)mp3_work;
}
